#!/usr/bin/env ruby
begin
  require 'plist'
  require 'erb'
  require 'pathname'
  require 'pp'
  require 'optparse'
rescue LoadError => e
  retry if require 'rubygems'
end

options = {
  :plugin_paths => [],
  :override_paths => [],
}
OptionParser.new do |opts|
  opts.banner = "Usage: plisttemplate.rb [options] [plugin base name]"

  opts.on("-o", "--override-path PATH", "Add path containing override plists") do |v|
    options[:override_paths] << v
  end

  opts.on("-p", "--plugin-search-path PATH", "Add path containing .qsplugin folders") do |v|
    options[:plugin_paths] << v
  end

  opts.on("-t", "--template FILEPATH", "Use specified template") do |v|
    options[:template] = v
  end

  opts.on("-v", "--[no-]verbose", "Run verbosely") do |v|
    options[:verbose] = v
  end
end.parse!
class PluginInfoLoader
  def initialize options
    @opt = options
    @bundles = {}
  end
  def load_overrides_for_id id
    result = {}
    @opt[:override_paths].each do |path|
      item = Pathname.new(path) + "#{id}.plist"
      p item
      item = item.realpath if item.exist?
      p item
      hash = item.exist? ? Plist::parse_xml(item.to_s) : nil
      if hash && hash.is_a?(Hash)
        $stderr.puts "Merging with overrides from #{item.to_s}" if @opt[:verbose]
        result.merge!(hash)
      end
    end
    result
  end
  def load_qsplugin path
    path = Pathname.new(path) unless path.is_a?(Pathname)
    path = path.realpath if path.exist?
    begin
      raise "Couldnt find #{path}!" unless path.directory?
      $stderr.puts "Loading #{path}" if @opt[:verbose]
      result = Plist::parse_xml(path + 'Contents/info.plist')
      id = result["CFBundleIdentifier"]
      if result && id
        result["QSModifiedDate"] = path.stat.mtime.strftime("%Y-%m-%d %H:%M:%S %z")
        result.merge!(self.load_overrides_for_id(id))
        $stderr.puts "More that one plugin with same ID, second at #{path}" if @opt[:verbose] && @bundles[id]
        @bundles[id] = result
        return result
      end
    rescue Exception => e
      $stderr.puts "Error: #{e}"
    end
    return nil
  end
  def load name
    @opt[:plugin_paths].map do |path|
      item = Pathname.new(path) + name
      item = item.realpath if item.exist?
      item = Pathname.new(path) + "#{name}.qsplugin" unless item.directory?
      item = item.realpath if item.exist?
      result = self.load_qsplugin(item)
      return result if result
    end
    return nil
  end
  def load_all
    result = []
    @opt[:plugin_paths].map do |path|
      item = Pathname.new(path)
      item = item.realpath if item.exist?
      plugins = Dir[item + "*.qsplugin"]
      result += plugins.map { |e| load_qsplugin(e) }.select {|e| e}.to_a
    end
    result
  end
end
class HashWithBinding
  def initialize hash
    @vals = {}
    hash.each_pair { |name, val| @vals[name] = val }
  end
  def method_missing(meth, *args, &blk)
    return @vals[meth.to_s]
  end
  def get_binding
    binding
  end
end
loader = PluginInfoLoader.new options
data = []
if ARGV.count > 0
  data = ARGV.map { |e| loader.load(e) }
  begin
    template = ERB.new(File.read(options[:template]))
  rescue Exception => e
    raise "To specify plugins on the command line you must specify a template with -t, error reading it: #{e}"
  end
  data.each do |input|
    puts "Writing output to #{input['CFBundleIdentifier'] + ".txt"}" if options[:verbose]
    File.open(input['CFBundleIdentifier'] + ".txt", 'w') {|f| f.write(template.result(HashWithBinding.new(input).get_binding)) }
  end
else
  data = loader.load_all
  data = data.map do |e|
    result = {}
    copy = %w[CFBundleIdentifier QSModifiedDate CFBundleName CFBundleVersion QSPlugIn QSRequirements CFBundleShortVersionString]
    e.each_pair do |k,v|
      result[k] = v if copy.index(k)
    end
    result
  end
  # maybe add a filter for compatible QS version at this point ?
  data = { :plugins => data }
  puts data.to_plist
end
